/*
 * File:   DBManager.h
 * Author: mdm
 *
 * Created on October 24, 2013, 2:57 PM
 */

#ifndef DBMANAGER_H
#define	DBMANAGER_H

#include <Configuration.h>
#include <Response.pb.h>

#include <tango.h>

#include <ctime>

#include <boost/tuple/tuple.hpp>
#include <boost/optional/optional.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/thread/mutex.hpp>

#include <soci/soci.h>
#include <soci/error.h>
#include <soci/row.h>
#include <soci/rowset.h>
#include <soci/boost-tuple.h>
#include <soci/boost-optional.h>
#include <soci/session.h>

namespace MetadataImporter_ns
{

class DBManager : public Tango::LogAdapter
{
public:
//------------------------------------------------------------------------------
//  [Public] Shared pointer typedef
//------------------------------------------------------------------------------
    typedef boost::shared_ptr<DBManager> SP;

protected:
//------------------------------------------------------------------------------
//  [Protected] Constructor destructor deleter
//------------------------------------------------------------------------------
    DBManager(Tango::DeviceImpl*, Configuration::SP);

    virtual ~DBManager();

    class Deleter;
    friend class Deleter;
    class Deleter
    {
        public:
            void operator()(DBManager* d) { delete d; }
    };

public:
//------------------------------------------------------------------------------
//  [Public] Class creation method
//------------------------------------------------------------------------------
    static DBManager::SP create(Tango::DeviceImpl*, Configuration::SP);

//------------------------------------------------------------------------------
//  [Public] Connection handling methods
//------------------------------------------------------------------------------
    virtual void connect() throw(soci::soci_error);

    virtual void disconnect();

//------------------------------------------------------------------------------
//  [Public] Queries methods
//------------------------------------------------------------------------------
    typedef boost::tuple< boost::optional<std::string>, boost::optional<std::string>,
        boost::optional<std::string> > InformationTuple;

    typedef std::vector< InformationTuple > InformationList;

    virtual InformationList retrieveInformation(std::string,
        std::string) throw(soci::soci_error);

    virtual std::tm retrieveLastTimestamp(std::string, std::string)
        throw(soci::soci_error);

    virtual void persistMetadata(std::string, std::string, const Response::Metadata&)
        throw(soci::soci_error, std::out_of_range);

protected:
//------------------------------------------------------------------------------
//  [Protected] Utilities method
//------------------------------------------------------------------------------
    virtual std::string composeInsertQuery(std::string, std::string,
        const Response::Metadata::Row&) throw(std::out_of_range);

//------------------------------------------------------------------------------
//  [Protected] Class variables
//------------------------------------------------------------------------------
    //Configuration shared pointer
    Configuration::SP m_configuration_sp;

    //Connection mutex
    boost::mutex m_connectionMutex;

    //Database connection scoped pointer
    boost::scoped_ptr<soci::session> m_session_sp;
};

}   //End of namespace

#endif	/* DBMANAGER_H */
