/*----- PROTECTED REGION ID(MetadataImporter.h) ENABLED START -----*/
//=============================================================================
//
// file :        MetadataImporter.h
//
// description : Include file for the MetadataImporter class
//
// project :     Metadata importer device server
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
// $Author:  $
//
// $Revision:  $
// $Date:  $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef MetadataImporter_H
#define MetadataImporter_H

#include <tango.h>

#include <Client.h>

/*----- PROTECTED REGION END -----*/	//	MetadataImporter.h

/**
 *  MetadataImporter class description:
 *    Metadata importer device server
 */

namespace MetadataImporter_ns
{
/*----- PROTECTED REGION ID(MetadataImporter::Additional Class Declarations) ENABLED START -----*/

//	Additional Class Declarations

/*----- PROTECTED REGION END -----*/	//	MetadataImporter::Additional Class Declarations

class MetadataImporter : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(MetadataImporter::Data Members) ENABLED START -----*/

//------------------------------------------------------------------------------
//  [Private] Class variables
//------------------------------------------------------------------------------
    //Configuration class shared pointer
    Configuration::SP m_configuration_sp;

    //Client class shared pointer
    Client::SP m_client_sp;

    //Tuple counter synchronization
    boost::mutex m_tupleCounterMutex;

    //Max port number allowed value for remote connection
    static const unsigned int MAX_REMOTE_PORT = 65535;

    //Max port number allowed value for local database
    static const unsigned int MAX_DATABASE_PORT = 65535;

    //Max time between remote server requests
    static const unsigned int MAX_REFRESH_TIME = 3600;

    //Max time between remote server request and response
    static const unsigned int MAX_TIMEOUT = 60;

/*----- PROTECTED REGION END -----*/	//	MetadataImporter::Data Members

//	Device property data members
public:
	//	CertificateFile:	Absolute path to certificate chain file
	string	certificateFile;
	//	RemoteHost:	Metadata exporter remote host
	string	remoteHost;
	//	RemotePort:	Metadata exporter remote port
	Tango::DevULong	remotePort;
	//	RemoteUsername:	Metadata exporter login username
	string	remoteUsername;
	//	RemotePassword:	Metadata exporter remote password
	string	remotePassword;
	//	RemoteSchema:	Metadata remote database schema
	string	remoteSchema;
	//	RemoteTable:	Metadata remote database table
	string	remoteTable;
	//	EnableSSL:	Enable or disable SSL connections
	Tango::DevBoolean	enableSSL;
	//	DatabaseHost:	Metadata local database host
	string	databaseHost;
	//	DatabasePort:	Metadata local database port
	Tango::DevULong	databasePort;
	//	DatabaseUsername:	Metadata local database username
	string	databaseUsername;
	//	DatabasePassword:	Metadata local database password
	string	databasePassword;
	//	DatabaseSchema:	Metadata local database schema
	string	databaseSchema;
	//	DatabaseTable:	Metadata local database table
	string	databaseTable;
	//	RefreshTime:	Remote database request period (seconds)
	Tango::DevULong	refreshTime;
	//	Timeout:	Connection timeout (seconds)
	Tango::DevULong	timeout;
	//	AutoStart:	Exec On command after init if state is not fault
	Tango::DevBoolean	autoStart;

//	Attribute data members
public:
	Tango::DevULong	*attr_TupleCounter_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	MetadataImporter(Tango::DeviceClass *cl,string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	MetadataImporter(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	MetadataImporter(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~MetadataImporter() {delete_device();};


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : MetadataImporter::read_attr_hardware()
	 *	Description : Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(vector<long> &attr_list);

/**
 *	Attribute TupleCounter related methods
 *	Description:
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
	virtual void read_TupleCounter(Tango::Attribute &attr);
	virtual bool is_TupleCounter_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method      : MetadataImporter::add_dynamic_attributes()
	 *	Description : Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();



//	Command related methods
public:
	/**
	 *	Command On related method
	 *	Description:
	 *
	 */
	virtual void on();
	virtual bool is_On_allowed(const CORBA::Any &any);
	/**
	 *	Command Off related method
	 *	Description:
	 *
	 */
	virtual void off();
	virtual bool is_Off_allowed(const CORBA::Any &any);
	/**
	 *	Command ResetCounter related method
	 *	Description:
	 *
	 */
	virtual void reset_counter();
	virtual bool is_ResetCounter_allowed(const CORBA::Any &any);


/*----- PROTECTED REGION ID(MetadataImporter::Additional Method prototypes) ENABLED START -----*/
//------------------------------------------------------------------------------
//  [Public] Counter method
//------------------------------------------------------------------------------
    virtual void incrementTupleCounter(long);

protected:
//------------------------------------------------------------------------------
//  [Protected] Utilities methods
//------------------------------------------------------------------------------
    virtual void checkIfFileExists(std::string)
        throw(std::invalid_argument);

/*----- PROTECTED REGION END -----*/	//	MetadataImporter::Additional Method prototypes
};

/*----- PROTECTED REGION ID(MetadataImporter::Additional Classes Definitions) ENABLED START -----*/

//	Additional Classes Definitions

/*----- PROTECTED REGION END -----*/	//	MetadataImporter::Additional Classes Definitions

}	//	End of namespace

#endif   //	MetadataImporter_H
